from .listing_inventory import ListingInventory
from .listing_property import ListingProperty
from .listing_request import Request, UpdateListingRequest, UpdateVariationImagesRequest

__all__ = [
    "ListingInventory",
    "ListingProperty",
    "Request",
    "UpdateListingRequest",
    "UpdateVariationImagesRequest",
]



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from etsyv3.enums import (
    ItemDimensionsUnit,
    ItemWeightUnit,
    ListingRequestState,
    ListingType,
    WhenMade,
    WhoMade,
)
from etsyv3.models.product import Product
from etsyv3.util import todict


class Request:
    def __init__(
        self,
        nullable: Optional[List[str]] = None,
        mandatory: Optional[List[str]] = None,
    ):
        self._nullable = nullable if nullable is not None else []
        self._mandatory = mandatory if mandatory is not None else []
        if not self.check_mandatory():
            raise ValueError

    def check_mandatory(self) -> bool:
        if self._mandatory is not None:
            for key in self._mandatory:
                try:
                    if self.__dict__[key] is None:
                        return False
                except Exception:
                    return False
        return True

    def get_nulled(self) -> List[str]:
        return [
            key
            for key, value in self.__dict__.items()
            if key in self._nullable and (value == [] or value == "" or value == 0)
        ]

    def get_dict(self) -> Any:
        nulled = self.get_nulled()
        return todict(self, nullable=nulled)


class CreateDraftListingRequest(Request):
    nullable = [
        "shipping_profile_id",
        "materials",
        "shop_section_id",
        "processing_min",
        "processing_max",
        "tags",
        "styles",
        "item_weight",
        "item_length",
        "item_width",
        "item_height",
        "item_weight_unit",
        "item_dimensions_unit",
        "production_partner_ids",
        "image_ids",
    ]
    mandatory = [
        "quantity",
        "title",
        "description",
        "price",
        "who_made",
        "when_made",
        "taxonomy_id",
    ]

    def __init__(
        self,
        quantity: Optional[int] = None,
        title: Optional[str] = None,
        description: Optional[str] = None,
        price: Optional[float] = None,
        who_made: Optional[WhoMade] = None,
        when_made: Optional[WhenMade] = None,
        taxonomy_id: Optional[int] = None,
        shipping_profile_id: Optional[int] = None,
        materials: Optional[List[str]] = None,
        shop_section_id: Optional[int] = None,
        processing_min: Optional[int] = None,
        processing_max: Optional[int] = None,
        tags: Optional[List[str]] = None,
        styles: Optional[List[str]] = None,
        item_weight: Optional[float] = None,
        item_length: Optional[float] = None,
        item_width: Optional[float] = None,
        item_height: Optional[float] = None,
        item_weight_unit: Optional[ItemWeightUnit] = None,
        item_dimensions_unit: Optional[ItemDimensionsUnit] = None,
        is_personalizable: Optional[bool] = None,
        personalization_is_required: Optional[bool] = None,
        personalization_char_count_max: Optional[int] = None,
        personalization_instructions: Optional[str] = None,
        production_partner_ids: Optional[int] = None,
        image_ids: Optional[List[int]] = None,
        is_supply: Optional[bool] = None,
        is_customizable: Optional[bool] = None,
        should_auto_renew: Optional[bool] = None,
        is_taxable: Optional[bool] = None,
        listing_type: Optional[ListingType] = None,
    ):
        self.quantity = quantity
        self.title = title
        self.description = description
        self.price = price
        self.who_made = who_made
        self.when_made = when_made
        self.taxonomy_id = taxonomy_id
        self.shipping_profile_id = shipping_profile_id
        self.materials = materials
        self.shop_section_id = shop_section_id
        self.processing_min = processing_min
        self.processing_max = processing_max
        self.tags = tags
        self.styles = styles
        self.item_weight = item_weight
        self.item_length = item_length
        self.item_width = item_width
        self.item_height = item_height
        self.item_weight_unit = item_weight_unit
        self.item_dimensions_unit = item_dimensions_unit
        self.is_personalizable = is_personalizable
        self.personalization_is_required = personalization_is_required
        self.personalization_char_count_max = personalization_char_count_max
        self.personalization_instructions = personalization_instructions
        self.production_partner_ids = production_partner_ids
        self.image_ids = image_ids
        self.is_supply = is_supply
        self.is_customizable = is_customizable
        self.should_auto_renew = should_auto_renew
        self.is_taxable = is_taxable
        self.listing_type = listing_type
        super().__init__(
            nullable=CreateDraftListingRequest.nullable,
            mandatory=CreateDraftListingRequest.mandatory,
        )


class UpdateListingRequest(Request):
    nullable: List[str] = [
        "materials",
        "shipping_profile_id",
        "shop_section_id",
        "item_weight",
        "item_length",
        "item_width",
        "item_height",
        "item_weight_unit",
        "item_dimensions_unit",
        "tags",
        "featured_rank",
        "production_partner_ids",
        "type",
    ]

    mandatory: List[str] = []

    def __init__(
        self,
        image_ids: Optional[List[str]] = None,
        title: Optional[str] = None,
        description: Optional[str] = None,
        materials: Optional[List[str]] = None,
        should_auto_renew: Optional[bool] = None,
        shipping_profile_id: Optional[int] = None,
        shop_section_id: Optional[int] = None,
        item_weight: Optional[float] = None,
        item_length: Optional[float] = None,
        item_width: Optional[float] = None,
        item_height: Optional[float] = None,
        item_weight_unit: Optional[ItemWeightUnit] = None,
        item_dimensions_unit: Optional[ItemDimensionsUnit] = None,
        is_taxable: Optional[bool] = None,
        taxonomy_id: Optional[int] = None,
        tags: Optional[List[str]] = None,
        who_made: Optional[WhoMade] = None,
        when_made: Optional[WhenMade] = None,
        featured_rank: Optional[int] = None,
        is_personalizable: Optional[bool] = None,
        personalization_is_required: Optional[bool] = None,
        personalization_char_count_max: Optional[int] = None,
        personalization_instructions: Optional[str] = None,
        state: Optional[ListingRequestState] = None,
        is_supply: Optional[bool] = None,
        production_partner_ids: Optional[List[int]] = None,
        listing_type: Optional[ListingType] = None,
    ):
        self.image_ids = image_ids
        self.title = title
        self.description = description
        self.materials = materials
        self.should_auto_renew = should_auto_renew
        self.shipping_profile_id = shipping_profile_id
        self.shop_section_id = shop_section_id
        self.item_weight = item_weight
        self.item_length = item_length
        self.item_width = item_width
        self.item_height = item_height
        self.item_weight_unit = item_weight_unit
        self.item_dimensions_unit = item_dimensions_unit
        self.is_taxable = is_taxable
        self.taxonomy_id = taxonomy_id
        self.tags = tags
        self.who_made = who_made
        self.when_made = when_made
        self.featured_rank = featured_rank
        self.is_personalizable = is_personalizable
        self.personalization_is_required = personalization_is_required
        self.personalization_char_count_max = personalization_char_count_max
        self.personalization_instructions = personalization_instructions
        self.state = state
        self.is_supply = is_supply
        self.production_partner_ids = production_partner_ids
        self.listing_type = listing_type
        super().__init__(nullable=UpdateListingRequest.nullable)


class UpdateListingInventoryRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = ["products"]

    def __init__(
        self,
        products: List[Product],
        price_on_property: Optional[List[int]] = None,
        quantity_on_property: Optional[List[int]] = None,
        sku_on_property: Optional[List[int]] = None,
    ):
        self.products = products
        self.price_on_property = price_on_property
        self.quantity_on_property = quantity_on_property
        self.sku_on_property = sku_on_property
        super().__init__(
            nullable=UpdateListingInventoryRequest.nullable,
            mandatory=UpdateListingInventoryRequest.mandatory,
        )

    @staticmethod
    def generate_request_from_inventory_response(
        response: Dict[str, Any]
    ) -> UpdateListingInventoryRequest:
        products = []
        for product in response["products"]:
            property_values = product["property_values"]
            for prop_val in property_values:
                prop_val.pop("scale_name", None)
                prop_val.pop("value_pairs", None)
            offerings = product["offerings"]
            for offering in offerings:
                offering.pop("is_deleted", None)
                offering.pop("offering_id", None)
                offering["price"] = (
                    offering["price"]["amount"] / offering["price"]["divisor"]
                )
            products.append(Product(product["sku"], property_values, offerings))
        price_on_property = response["price_on_property"]
        quantity_on_property = response["quantity_on_property"]
        sku_on_property = response["sku_on_property"]
        return UpdateListingInventoryRequest(
            products, price_on_property, quantity_on_property, sku_on_property
        )


class UpdateVariationImagesRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = []

    def __init__(self, variation_images: List[Dict[str, Any]]) -> None:
        self.variation_images = variation_images
        super().__init__(
            nullable=UpdateVariationImagesRequest.nullable,
            mandatory=UpdateVariationImagesRequest.mandatory,
        )

    @staticmethod
    def generate_request_from_variation_images_response(
        response: Dict[str, Any]
    ) -> UpdateVariationImagesRequest:
        variation_images = response["results"]
        for variation_image in variation_images:
            variation_image.pop("value", None)
        return UpdateVariationImagesRequest(variation_images)

class UpdateListingImageIDRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = ["listing_image_id"]

    def __init__(
        self,
        listing_image_id: int,
        rank: Optional[int] = None,
        overwrite: Optional[bool] = None,
        is_watermarked: Optional[bool] = None,
        alt_text: Optional[str] = None,
    ) -> None:

        self.listing_image_id = listing_image_id
        self.rank = rank
        self.overwrite = overwrite
        self.is_watermarked = is_watermarked
        self.alt_text = alt_text

        super().__init__(
            nullable=UpdateListingImageIDRequest.nullable,
            mandatory=UpdateListingImageIDRequest.mandatory,
        )


class UpdateListingPropertyRequest(Request):
    nullable: List[str] = []

    mandatory: List[str] = [
        "value_ids",
        "values",
    ]

    def __init__(
        self,
        value_ids: Optional[List[int]] = None,
        values: Optional[List[str]] = None,
        scale_id: Optional[int] = None,
    ):
        self.value_ids = value_ids
        self.values = values
        self.scale_id = scale_id
        super().__init__(
            nullable=UpdateListingPropertyRequest.nullable,
            mandatory=UpdateListingPropertyRequest.mandatory,
        )



from typing import Any, Dict, List, Optional

from etsyv3.models import Request


class FileRequest(Request):
    def __init__(
        self,
        nullable: Optional[List[str]] = None,
        mandatory: Optional[List[str]] = None,
    ) -> None:
        self.file: Dict[str, Any] = self.file if self.file is not None else None
        self.data: Dict[str, Any] = self.data if self.data is not None else None
        super().__init__(nullable=nullable, mandatory=mandatory)

    @staticmethod
    def generate_bytes_from_file(file: str) -> bytes:
        with open(file, "rb") as f:
            f_bytes = f.read()
        return f_bytes


class UploadListingImageRequest(FileRequest):
    nullable: List[str] = ["file"]
    mandatory: List[str] = []

    def __init__(
        self,
        image_bytes: bytes,
        listing_image_id: Optional[int] = None,
        rank: Optional[int] = None,
        overwrite: Optional[bool] = None,
        is_watermarked: Optional[bool] = None,
        alt_text: Optional[str] = None,
    ) -> None:
        self.file = {"image": image_bytes}
        self.data = {
            "listing_image_id": listing_image_id,
            "rank": rank,
            "overwrite": overwrite,
            "is_watermarked": is_watermarked,
            "alt_text": alt_text,
        }

        super().__init__(
            nullable=UploadListingImageRequest.nullable,
            mandatory=UploadListingImageRequest.mandatory,
        )


class UploadListingFileRequest(FileRequest):
    nullable: List[str] = ["file"]
    mandatory: List[str] = []

    def __init__(
        self,
        file_bytes: bytes,
        listing_file_id: Optional[int] = None,
        name: Optional[str] = None,
        rank: Optional[int] = None,
    ) -> None:
        self.file = {"file": (name, file_bytes, "multipart/form-data")}
        self.data = {"listing_file_id": listing_file_id, "rank": rank, "name": name}

        super().__init__(
            nullable=UploadListingFileRequest.nullable,
            mandatory=UploadListingFileRequest.mandatory,
        )


class UploadListingVideoRequest(FileRequest):
    nullable: List[str] = ["video"]
    mandatory: List[str] = []

    def __init__(
        self,
        video_bytes: bytes,
        listing_video_id: Optional[int] = None,
        name: Optional[str] = None,
    ) -> None:
        self.file = {"video": video_bytes}
        self.data = {
            "listing_video_id": listing_video_id,
            "name": name,
        }

        super().__init__(
            nullable=UploadListingVideoRequest.nullable,
            mandatory=UploadListingVideoRequest.mandatory,
        )



from typing import List, Optional

from etsyv3.models.listing_request import Request


class CreateShopSectionRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = ["title"]

    def __init__(self, title: str):
        self.title = title

        super().__init__(
            nullable=CreateShopSectionRequest.nullable,
            mandatory=CreateShopSectionRequest.mandatory,
        )


class UpdateShopSectionRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = ["title"]

    def __init__(self, title: str):
        self.title = title

        super().__init__(
            nullable=UpdateShopSectionRequest.nullable,
            mandatory=UpdateShopSectionRequest.mandatory,
        )


class UpdateShopRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = []

    def __init__(
        self,
        title: Optional[str] = None,
        announcement: Optional[str] = None,
        sale_message: Optional[str] = None,
        digital_sale_message: Optional[str] = None,
        policy_additional: Optional[str] = None,
    ):
        self.title = title
        self.announcement = announcement
        self.sale_message = sale_message
        self.digital_sale_message = digital_sale_message
        self.policy_additional = policy_additional

        super().__init__(
            nullable=UpdateShopRequest.nullable,
            mandatory=UpdateShopRequest.mandatory,
        )



from typing import List, Optional, Union

from etsyv3.enums import ShippingProvider
from etsyv3.models.listing_request import Request


class CreateReceiptShipmentRequest(Request):
    nullable: List[str] = []
    mandatory: List[str] = []

    def __init__(
        self,
        tracking_code: Optional[str] = None,
        carrier_name: Optional[Union[ShippingProvider, str]] = None,
        send_bcc: Optional[bool] = None,
        note_to_buyer: Optional[str] = None,
    ):
        self.tracking_code = tracking_code
        self.carrier_name = carrier_name
        self.send_bcc = send_bcc
        self.note_to_buyer = note_to_buyer

        super().__init__(
            nullable=CreateReceiptShipmentRequest.nullable,
            mandatory=CreateReceiptShipmentRequest.mandatory,
        )


class UpdateShopReceiptRequest(Request):
    nullable: List[str] = [
        "was_shipped",
        "was_paid",
    ]
    mandatory: List[str] = []

    def __init__(
        self,
        was_shipped: Optional[bool] = None,
        was_paid: Optional[bool] = None,
    ):
        self.was_shipped = was_shipped
        self.was_paid = was_paid

        super().__init__(
            nullable=UpdateShopReceiptRequest.nullable,
            mandatory=UpdateShopReceiptRequest.mandatory,
        )



from enum import Enum


class ShippingProvider(Enum):
    FOUR_PX_WORLDWIDE_EXPRESS = "4px"
    A1POST = "a1post"
    ABF_FREIGHT = "abf"
    ACS_COURIER = "acscourier"
    AEROFLASH = "AEROFLASH"
    AFGHAN_POST = "afghan-post"
    AMAZON_LOGISTICS_UK = "amazon-uk-api"
    AMAZON_LOGISTICS_US = "amazon"
    AN_POST = "an-post"
    ANGUILLA_POSTAL_SERVICE = "anguilla-post"
    APC_POSTAL_LOGISTICS = "apc"
    ARAMEX = "aramex"
    ASENDIA_UK = "asendia-uk"
    ASENDIA_USA = "asendia-usa"
    AUSTRALIA_POST = "australia-post"
    AUSTRIAN_POST = "austrian-post"
    AUSTRIAN_POST_REGISTERED = "austrian-post-registered"
    BAHRAIN_POST = "bahrain-post"
    BANGLADESH_POST_OFFICE = "bangladesh-post"
    BELGIUM_POST_DOMESTIC = "bpost"
    BELGIUM_POST_INTERNATIONAL = "bpost-international"
    BELPOSHTA = "belpost"
    BH_POSTA = "bh-posta"
    BLUE_DART = "bluedart"
    BOTSWANAPOST = "botswanapost"
    BRUNEI_POSTAL_SERVICES = "brunei-post"
    BULGARIAN_POSTS = "bgpost"
    CAMBODIA_POST = "cambodia-post"
    CANADA_POST = "canada-post"
    CANPAR_COURIER = "canpar"
    CESKA_POSTA = "ceska-posta"
    CHINA_EMS = "china-ems"
    CHINA_POST = "china-post"
    CHIT_CHATS = "chitchats"
    CHRONOPOST_FRANCE = "chronopost-france"
    CHRONOPOST_PORTUGAL = "chronopost-portugal"
    CHUNGHWA_POST = "taiwan-post"
    CITY_LINK = "city-link"
    COLISSIMO = "colissimo"
    COLLECT_PLUS = "collectplus"
    CORREIOS_DE_BRASIL = "brazil-correios"
    CORREIOS_DE_MACAU = "correios-macau"
    CORREIOS_DE_PORTUGAL_CTT = "portugal-ctt"
    CORREO_ARGENTINO_DOMESTIC = "correo-argentino"
    CORREO_ARGENTINO_INTERNATIONAL = "correo-argentino-intl"
    CORREO_URUGUAYO = "correo-uruguayo"
    CORREOS_ESPANA = "spain-correos-es"
    CORREOS_CHILE = "correos-chile"
    CORREOS_DE_MEXICO = "correos-de-mexico"
    CORREOS_DE_COSTA_RICA = "correos-de-costa-rica"
    CORREOS_DEL_ECUADOR = "correos-ecuador"
    COURIER_POST = "courierpost"
    COURIERS_PLEASE = "couriers-please"
    CYPRUS_POST = "cyprus-post"
    DELTEC_COURIER = "deltec-courier"
    DEUTSCHE_POST = "deutsch-post"
    DHL_BENELUX = "dhl-benelux"
    DHL_EXPRESS = "dhl"
    DHL_GERMANY = "dhl-germany"
    DHL_GLOBAL_MAIL = "dhl-global-mail"
    DHL_NETHERLANDS = "dhl-nl"
    DHL_PARCEL_NL = "dhlparcel-nl"
    DHL_POLSKA = "dhl-poland"
    DHL_SPAIN_DOMESTIC = "dhl-es"
    DHL_ECOMMERCE = "dhl-global-mail-asia"
    DIRECT_LINK = "directlink"
    DPD = "dpd"
    DPD_GERMANY = "dpd-de"
    DPD_POLSKA = "dpd-poland"
    DPD_UK = "dpd-uk"
    DTDC_INDIA = "dtdc"
    EC_FIRSTCLASS = "ec-firstclass"
    EGYPT_POST = "egypt-post"
    EL_CORREO = "el-correo"
    ELTA_COURIER = "elta-courier"
    EMPOST = "emirates-post"
    EMPRESA_DE_CORREOS_DE_BOLIVIA = "correos-bolivia"
    ESTAFETA = "estafeta"
    ESTES = "estes"
    ESTONIAN_POST = "estonian-post"
    ETHIOPIAN_POSTAL_SERVICE = "ethiopian-post"
    EVERGREEN = "evergreen"
    FASTWAY_AUSTRALIA = "fastway-au"
    FASTWAY_COURIERS = "fastway-ireland"
    FASTWAY_NEW_ZEALAND = "fastway-nz"
    FASTWAYS_COURIERS_SOUTH_AFRICA = "fastway-za"
    FEDEX = "fedex"
    FEDEX_UK_DOMESTIC = "fedex-uk"
    FIRST_FLIGHT_COURIERS = "first-flight"
    FLASH_COURIER = "flash-courier"
    FREIGHTQUOTE_BY_C_H_ROBINSON = "freightquote"
    GATI_KWE = "gati-kwe"
    GHANA_POST = "ghana-post"
    GLOBEGISTICS = "globegistics"
    GLS = "gls"
    GREYHOUND = "greyhound"
    GUERNSEY_POST = "guernsey-post"
    HAY_POST = "hay-post"
    HELLENIC_POST = "hellenic-post"
    HERMES = "hermes-de"
    HERMES_ITALY = "hermes-it"
    HERMES_UK = "hermes"
    HONG_KONG_POST = "hong-kong-post"
    HRVATSKA_POSTA = "hrvatska-posta"
    I_PARCEL = "i-parcel"
    INDIA_POST = "india-post"
    INDIA_POST_INTERNATIONAL = "india-post-int"
    INTERLINK_EXPRESS = "interlink-express"
    INTERNATIONAL_SEUR = "international-seur"
    IPOSTEL = "ipostel"
    IRAN_POST = "iran-post"
    ISLANDSPOSTUR = "islandspostur"
    ISLE_OF_MAN_POST_OFFICE = "isle-of-man-post"
    ISRAEL_POST = "israel-post"
    ISRAEL_POST_DOMESTIC = "israel-post-domestic"
    JAMAICA_POST = "jamaica-post"
    JAPAN_POST = "japan-post"
    JERSEY_POST = "jersey-post"
    JORDAN_POST = "jordan-post"
    KAZPOST = "kazpost"
    KOREA_POST = "kpost"
    KOREA_POST_EMS = "korea-post"
    KUEHNE_NAGEL = "kn"
    LA_POSTE = "la-poste-colissimo"
    LA_POSTE_MONACO = "poste-monaco"
    LA_POSTE_DU_SENEGAL = "poste-senegal"
    LA_POSTE_TUNISIENNE = "poste-tunisienne"
    LANDMARK_GLOBAL = "landmark-global"
    LASERSHIP = "lasership"
    LATVIJAS_PASTS = "latvijas-pasts"
    LIBANPOST = "libanpost"
    LIETUVOS_PASTAS = "lietuvos-pastas"
    MAGYAR_POSTA = "magyar-posta"
    MAKEDONSKA_POSTA = "makedonska-posta"
    MALAYSIA_POS_DAFTAR = "malaysia-post-posdaftar"
    MALDIVES_POST = "maldives-post"
    MALTAPOST = "maltapost"
    MAURITIUS_POST = "mauritius-post"
    MONDIAL_RELAY = "mondialrelay"
    MRW = "mrw-spain"
    MULTIPACK = "mexico-multipack"
    MYHERMES_UK = "myhermes-uk"
    NACEX = "nacex-spain"
    NEW_ZEALAND_POST = "new-zealand-post"
    NEXIVE = "tntpost-it"
    NIEUWE_POST_NEDERLANDSE_ANTILLEN_PNA = "nieuwe-post-nederlandse-antillen-pna"
    NIGERIAN_POSTAL_SERVICE = "nipost"
    NOVA_POSHTA = "nova-poshta"
    OCA = "oca-ar"
    OPEK = "opek"
    OMAN_POST = "oman-post"
    ONTRAC = "ontrac"
    OPT = "opt"
    OPT_DE_NOUVELLE_CALEDONIE = "opt-nouvelle-caledonie"
    OTHER = "other"
    PAKISTAN_POST = "pakistan-post"
    PARCELFORCE_WORLDWIDE = "parcel-force"
    POCZTA_POLSKA = "poczta-polska"
    POS_INDONESIA = "pos-indonesia"
    POS_INDONESIA_INTERNATIONAL = "pos-indonesia-int"
    POS_MALAYSIA = "malaysia-post"
    POST_ARUBA = "post-aruba"
    POST_FIJI = "post-fiji"
    POST_LUXEMBOURG = "post-luxembourg"
    POSTNL_DOMESTIC = "postnl"
    POSTNL_INTERNATIONAL = "postnl-international"
    POSTNL_INTERNATIONAL_3S = "postnl-3s"
    POSTNORD = "danmark-post"
    POSTNORD_LOGISTICS = "postnord"
    POSTA = "posta"
    POSTA_KENYA = "posta-kenya"
    POSTA_MOLDOVEI = "posta-moldovei"
    POSTA_ROMANA = "posta-romana"
    POSTA_SHQIPTARE = "posta-shqiptare"
    POSTA_SLOVENIJE = "posta-slovenije"
    POSTA_SRBIJE = "posta-srbije"
    POSTA_UGANDA = "posta-uganda"
    POSTE_ITALIANE = "poste-italiane"
    POSTE_ITALIANE_PACCOCELERE = "poste-italiane-paccocelere"
    POSTE_MAROC = "poste-maroc"
    POSTEN_AB = "sweden-posten"
    POSTEN_NORGE = "posten-norge"
    POSTI = "posti"
    POSTMATES = "postmates"
    PTT_POSTA = "ptt-posta"
    PUROLATOR = "purolator"
    QATAR_POST = "qatar-post"
    RED_EXPRESS = "red-express"
    REDPACK = "mexico-redpack"
    ROYAL_MAIL = "royal-mail"
    RL_CARRIERS = "rl-carriers"
    RPX_INDONESIA = "rpx"
    RUSSIAN_POST = "russian-post"
    S_F_INTERNATIONAL = "sfb2c"
    SAFEXPRESS = "safexpress"
    SAGAWA = "sagawa"
    SAUDI_POST = "saudi-post"
    SDA_EXPRESS_COURIER = "italy-sda"
    SELEKTVRACHT = "selektvracht"
    SENDA_EXPRESS = "mexico-senda-express"
    SENDLE = "sendle"
    SERPOST = "serpost"
    SEUR_ESPANA_DOMESTICO = "spanish-seur"
    SEUR_PORTUGAL_DOMESTICO = "portugal-seur"
    SF_EXPRESS = "sf-express"
    SINGAPORE_POST = "singapore-post"
    SINGAPORE_SPEEDPOST = "singapore-speedpost"
    SIODEMKA = "siodemka"
    SKYNET_MALAYSIA = "skynet-malaysia"
    SKYNET_WORDWIDE_EXPRESS = "skynetworldwide"
    SKYNET_WORLDWIDE_EXPRESS = "skynetworldwide"
    SLOVENSKA_POSTA = "slovenska-posta"
    SOUTH_AFRICA_POST_OFFICE = "sapo"
    STALLION_EXPRESS = "stallionexpress"
    STARTRACK = "star-track"
    SWISS_POST = "swiss-post"
    TA_Q_BIN_HONG_KONG = "taqbin-hk"
    TA_Q_BIN_JAPAN = "taqbin-jp"
    TA_Q_BIN_MALAYSIA = "taqbin-my"
    TA_Q_BIN_SINGAPORE = "taqbin-sg"
    TGX = "tgx"
    THAILAND_POST = "thailand-post"
    TNT = "tnt"
    TNT_AUSTRALIA = "tnt-au"
    TNT_FRANCE = "tnt-fr"
    TNT_ITALIA = "tnt-it"
    TNT_UK = "tnt-uk"
    TOLL_GLOBAL_EXPRESS = "toll-global-express"
    TOLL_PRIORITY = "toll-priority"
    TTPOST = "ttpost"
    UK_MAIL = "uk-mail"
    UKRPOSHTA = "ukrposhta"
    UPS = "ups"
    UPS_FREIGHT = "ups-freight"
    USHIP = "uship"
    USPS = "usps"
    VANUATU_POST = "vanuatu-post"
    VIETNAM_POST = "vnpost"
    VIETNAM_POST_EMS = "vnpost-ems"
    WHISTL = "whistl"
    XEND = "xend"
    YAKIT = "yakit"
    YANWEN = "yanwen"
    YEMEN_POST = "yemen-post"
    YODEL = "yodel"
    YODEL_INTERNATIONAL = "yodel-international"
    YRC_FREIGHT = "yrc"
    ZAMPOST = "zampost"
    ZIMPOST = "ZIMPOST"
